from enum import Enum
from gzip import GzipFile
from typing import Tuple
from nibabel import Nifti1Header, Nifti2Header

from .util import PartialBuffer, probe
from .exceptions import NotNiftiException

__version__="0.0.2"

NIFTI1_MAGIC_BYTES = 348
NIFTI2_MAGIC_BYTES = 540


class Endianness(Enum):
    LITTLE = 'little'
    BIG = 'big'


class NiftiTypes(Enum):
    NIFTI1 = 1
    NIFTI2 = 2


def check_bytes_magic(b: bytes) -> Tuple[Endianness, NiftiTypes]:

    magic_number = int.from_bytes(b[0:4], "little")
    if magic_number == NIFTI1_MAGIC_BYTES:
        return (Endianness.LITTLE, NiftiTypes.NIFTI1)
    if magic_number == NIFTI2_MAGIC_BYTES:
        return (Endianness.LITTLE, NiftiTypes.NIFTI2)

    magic_number = int.from_bytes(b[0:4], "big")
    if magic_number == NIFTI1_MAGIC_BYTES:
        return (Endianness.BIG, NiftiTypes.NIFTI1)
    if magic_number == NIFTI2_MAGIC_BYTES:
        return (Endianness.BIG, NiftiTypes.NIFTI2)
    
    raise NotNiftiException


def byte_to_header(b: bytes, endianness: Endianness, nifti_type: NiftiTypes) -> Nifti1Header:
    header_cls = None
    offset = None
    if nifti_type == NiftiTypes.NIFTI1:
        header_cls = Nifti1Header
        offset = NIFTI1_MAGIC_BYTES
    if nifti_type == NiftiTypes.NIFTI2:
        header_cls = Nifti2Header
        offset = NIFTI2_MAGIC_BYTES
    if header_cls is None:
        raise RuntimeError
    return header_cls(b[0:offset], endianness.value)
    


def get_nii_header(url: str, headers=None) -> Nifti1Header:
    output = probe(url, (0, 600), headers)
    try:
        endianness, niftitype = check_bytes_magic(output)
        return byte_to_header(output, endianness, niftitype)
    except NotNiftiException as e:
        pass

    partial_buffer = PartialBuffer(url, headers)
    fh = GzipFile(fileobj=partial_buffer)
    output = fh.peek(600)
    fh.close()

    endianness, niftitype = check_bytes_magic(output)
    return byte_to_header(output, endianness, niftitype)



import requests

def probe(url:str, range=(0, 200), headers=None) -> bytes:
    _headers = {
        **({} if headers is None else headers),
        "Range": f"bytes={'-'.join(str(r) for r in range)}"
    }
    
    resp = requests.get(url, headers=_headers)
    if resp.status_code >= 400:
        raise RuntimeError(f"probe error: {str(resp.status_code)}")
    no_byte_requested = range[1] - range[0] + 1
    no_byte_received = len(resp.content)
    assert no_byte_requested == no_byte_received, f'expect number of bytes received {no_byte_received} == requested {no_byte_requested}'
    return bytes(resp.content)


class PartialBuffer:
    def __init__(self, url: str, headers=None) -> None:
        self.url= url
        self.marker = 0
        self.headers = headers

    def read(self, size=-1):
        start_marker = self.marker
        current_marker = start_marker + size
        self.marker = current_marker
        return probe(self.url, (start_marker, current_marker - 1), self.headers)
        




import os
from setuptools import setup, find_packages
from telenib import __version__

ROOT_DIR = os.path.dirname(os.path.abspath(__file__))

with open(os.path.join(ROOT_DIR, "README.md"), "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="telenib",
    version=__version__,
    author="Xiao Gui",
    author_email="xgui3783@gmail.com",
    description="Fetch metadata from remote (https) nifti (gzipped or not).",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(
        include=["telenib"]
    ),
    install_requires=[
        "requests>=2.26.0",
        'nibabel>=4.0.1',
    ],
    url="https://github.com/xgui3783/telenib",
)
